function EHSoutput = EHS_Fnew(inputdat, opts, Fnew_opts)
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% EHS_Fnew    estimates new water fractions by ensemble hydrograph separation, with optional point weights
%
% Version 1.3  build 2020.06.21
% Author: James Kirchner and Julia Knapp, ETH Zurich
% This code was translated from R to MATLAB by Julia Knapp and tested for
% MATLAB R2018b and R2019b
%
% Copyright (C) 2020 ETH Zurich and James Kirchner
% Public use of this script is permitted under GNU General Public License 3 (GPL3); for details see <https://www.gnu.org/licenses/>
% 
%
% READ THIS CAREFULLY:
% ETH Zurich, James Kirchner and Julia Knapp make ABSOLUTELY NO WARRANTIES OF ANY KIND, including NO WARRANTIES, expressed or implied, that this software is
%    free of errors or is suitable for any particular purpose.  Users are solely responsible for determining the suitability and
%    reliability of this software for their own purposes.
%
% ALSO READ THIS:
% These scripts implement the ensemble hydrograph separation approach as presented in J.W. Kirchner, "Quantifying new water fractions
%    and transit time distributions using ensemble hydrograph separation: theory and benchmark tests", Hydrology and Earth System Sciences, 
%    23, 303-349, 2019.  These scripts are further described in J.W. Kirchner and J.L.A. Knapp, "Matlab and R scripts for ensemble hydrograph
%    separation", Hydrology and Earth System Sciences.  
%    Users should cite both of these papers (the first for the method, and the second for the scripts)


% The equation numbers in the scripts refer to the corresponding equations in Kirchner (2019).  The equations may differ in detail.  For example,
% in these scripts, k ranges from 1 to m+1 instead of 0 to m due to the array indexing conventions of the R language.
% Also, as described below, in calculating TTD's the counts of above-threshold precipitation n.xk and n.xkxl have been
% replaced by counts of usable inputs, denoted u.xk and u.xkxl.  Usable inputs either have precipitation tracer measurements,
% or have below-threshold precipitation so that one can assume that the tracer input was negligible.  The equations are adjusted accordingly.



%% estimate new water fractions             
% takes as input:
% - Cp           vector of tracer concentration in precipitation (at equally spaced points in time)
% - Cq           vector of tracer concentration in streamflow (assumed to be a vector at the same time points at Cp)
% - P            vector of precipitation rate (over the same time intervals as Cp)
% - Q            optional vector of discharge (in same units as precipitation)
% - p_threshold  threshold precipitation rate below which tracer inputs are ignored
% - vol_wtd      logical flag indicating whether volume-weighted new water fractions are desired (TRUE) or not (FALSE)
% - ptfilter     optional vector of flags (1 and 0 = TRUE and FALSE) indicating whether individual time steps should be included in the analysis
% - robust       if TRUE (1), Fnew is estimated by iteratively reweighted least squares (a robust estimation method)
% - ser_corr     if TRUE (1), standard error estimates account for serial correlation in residuals

% returns the following outputs
% - QpFnew        new water fraction for time steps with precipitation
% - QpFnew_se     standard error of QpFnew
% - QFnew         new water fraction for all time steps, including periods below rainfall threshold
% - QFnew_se      standard error of QFnew
% - PFnew         forward new water fraction (fraction of precipitation that becomes streamflow in the same time step)
% - PFnew_se      standard error of forward Fnew

% Cp and Cq can have missing values (NaN).
% P must exist, and must be non-negative, for all time steps that pass through the point filter (ptfilter==TRUE).
% Q must either be NaN for all time steps that pass through the point filter (ptfilter==TRUE),
%     or must exist and be be non-negative for all of these points.

% If Q values are NaN, then the forward new water fraction will be NaN.
% If Q values are NaN and vol_wtd==TRUE, a fatal error will result.

% Users should be aware that setting robust = TRUE can potentially give misleading results,  
% if the (many) low-flow points contain relatively small amounts of new water, and 
% the (few) high-flow points have much larger new-water proportions.  
% In such cases -- which are common -- the high-flow points contain most of the information 
% about new-water contributions to streamflow, but they may be ignored by robust fits because 
% they do not follow the same pattern as the (much more numerous) low-flow points.
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin < 2       % create opts strcutrue
    opts = [];
end
if nargin < 3   % create Fnew_opts strcutrue
    Fnew_opts = [];
end
% extract data from structures
Cp = inputdat.Cp;
Cq = inputdat.Cq;
P = inputdat.p;

%%%%%%%%%%
% set default values
% inputdat
if isfield(inputdat,'q')   Q = inputdat.q;
else     Q = nan(size(Cp));     end

% opts
if isfield(opts,'ser_corr')     ser_corr = opts.ser_corr;
else     ser_corr = 1; end

if isfield(opts,'robust')     robust = opts.robust;
else     robust = 1;  end

if isfield(opts,'vol_wtd')     vol_wtd = opts.vol_wtd;
else     vol_wtd = 0;   end

if isfield(opts,'p_thresh')     p_threshold = opts.p_thresh;
else     p_threshold = 0;   end

% EHS opts
if isfield(Fnew_opts,'ptfilter')     ptfilter = Fnew_opts.ptfilter;
else    ptfilter = ones(size(P));    end



    
n = length(Cp);
    
% check input data
if ((length(Cq) ~= n) || (length(P) ~= n) || (length(ptfilter) ~= n) || (length(Q) ~= n)) 
    error('Fatal error in EHS_Fnew: Cq, Cp, P, Q, and ptfilter must have same length!')
end

% coerce point filter to logical (in case it was supplied as a numeric vector)
ptfilter = logical(ptfilter);

if sum(isnan(P(ptfilter))) > 0 % if P is NaN for any point that passes through the point filter
   error('Fatal error in EHS_Fnew: P must not be missing!')     % throw an error
end

Qfilt = Q(ptfilter);                        % subset Q to include only points that pass through the point filter
if sum(~isnan(Qfilt)) > 0                 % if any of these Q's are non-NaN
    if min(Qfilt) <0                        % then if any are negative
        error('Fatal error in EHS_Fnew: Q must be >= 0!')     % throw an error
    end
    if sum(isnan(Qfilt))   >0                % and if any are NaN
        error('Fatal error in EHS_Fnew: missing values in Q!')     % throw an error
    end
end

if sum(isnan(Qfilt))>0 && vol_wtd        % if Q values are missing but we're trying to volume-weight Fnew
    error('Fatal error in EHS_Fnew: missing Q values in volume-weighted Fnew!')     % throw an error
end

% for volume-weighted new water fractions, we need to use discharge as weights
% otherwise we use a vector of 1's as dummy weights
if vol_wtd 
    wt = Q;
else 
    wt = ones(size(Cp));
end
 

% save a copy of the original weights, for future use in rescaling results
% we need this copy if we do robust estimation, because that will change values of wt
orig_wt = wt;
    
    
% for robust estimation, first exclude extreme values of Cp and Cq (since they could potentially create  
% x's with leverage that is too strong for robust estimation to overcome)
if robust
    Cp = outlier_filter(Cp);
    Cq = outlier_filter(Cq);
end
    

% create Cq.ref (Cq, lagged by 1) -- reference value of Cq
Cq_ref = [NaN;Cq(1:end-1)];     % equation 9

Cp(~ptfilter) = NaN;    % apply point filter to precipitation concentrations (NaN values of Cp will block evaluation of these time steps)
Cq(~ptfilter) = NaN;    % apply point filter to discharge concentrations
 
% replace Cp with NaN for P less than p_threshold
Cp(P < p_threshold) = nan;

% now we set up the variables x and y that we will regress to estimate new water fractions
% create y from Cq and Cq_ref
y = Cq - Cq_ref;         % equation 9

% create x from Cp and Cq_ref
x = Cp - Cq_ref;         % equation 9

% mask values of x, y, and wt for which each other have missing values
y(isnan(x) | isnan(wt)) = nan;
x(isnan(y)) = nan;    %this also masks any values that were NaN in the original y
wt(isnan(x)) = nan;   %this also masks any values that were NaN in the original y

if robust
    % robust slope estimation with IRLS
    bb = IRLS(y(~isnan(y)), x(~isnan(x)),  wt(~isnan(wt)));  % robust fit via IRLS
    beta_hat = bb.Coefficients.Estimate(2);
else 
    % if we do not want to do robust estimation
    % calculated weighted covariance matrix
    var_x = wtd_cov(x,x,wt);

    % create weighted xy covariance vector
    covar_xy = wtd_cov(x,y,wt);

    % now we calculate the regression slope
    beta_hat =covar_xy./ var_x;   % equation 16
end 
    
% calculating the standard error involves several steps
% first we need the effective sample size

% effective sample size, accounting for uneven weighting
% this will equal the non-missing n, if the weights are all equal
n_eff =(nansum(wt).^ 2) / nansum(wt.* wt);    % equation 19 (with general weight vector, not necessarily Q)

% Be aware that conventional weighted least squares does not use the effective sample size as calculated here,
% but rather just the number of non-missing n.  Conventional weighted least squares is designed specifically for
% situations where different points have different expected uncertainties, and the weights equal the reciprocals
% of the estimated variances (square of uncertainty) of each point.  The procedure followed here is for situations
% where the expected uncertainties of the points are the same, but some deserve more weight (for example, so that
% one obtains an unbiased estimate of a total flux).  In such cases, conventional weighted least squares will 
% underestimate the true uncertainty in the fitted coefficients, and the approach followed here will not.


if ser_corr
      % to account for serial correlation we need the residuals
      mean_y = nansum(y.*wt)./nansum(wt);  % weighted means
      mean_x = nansum(x.*wt)./nansum(wt);
      e = y - mean_y - beta_hat * (x - mean_x); % these are the residuals
      
      xe = e(~isnan(e));      % collapse the residual vector to retain only non-missing values (but keep them in order)
      xwt = wt(~isnan(e));    % collapse the weight vector too

      % 'Collapsing' the residuals and their weights, like this, is necessary because when there are lots of gaps
      % (for example, when we only analyze high-flow points), there may be very few adjacent pairs of residuals, which
      % will have undue influence on the serial correlation coefficient (calculated below).  Accurately estimating
      % serial correlation (and its effects on uncertainties) requires that we take account of correlations between all
      % the residuals and their lags (that is, the previous residuals), whether or not those pairs occur during adjacent time steps.
      
      % calculate the serial correlation coefficient (with geometric mean weights)
      r_sc = wtd_cor( xe, [NaN;xe(1:end-1)], sqrt(xwt.*[NaN;xwt(1:end-1)]));
      
      % effective degrees of freedom, accounting for serial correlation in residuals
      % Compare with equation 13 of K2019 -- here we apply the serial correlation 
      % correction to the degrees of freedom rather than sample size
      df_eff = (n_eff-2) * (1.0 - r_sc)./ (1.0 + r_sc);  
else
    df_eff = n.eff - 2;  % if ser_corr == FALSE
end

% the weighted correlation between x and y
r_xy = wtd_cor(x,y,wt);

% now we have all the ingredients for the standard error
if df_eff>0
    se = (abs(beta_hat) / sqrt(df_eff)).* sqrt((1 / r_xy.^ 2) - 1);    % equations 11, 15, 20
else 
    se = NaN;   % if we have negative degrees of freedom, standard error is NaN
end

%  calculate conversion factors
% this conversion factor accounts for rainless periods (equations 14, 15, 18)
sumwt = nansum(orig_wt(ptfilter));  % number of time steps, or their total discharge
if sumwt == 0
    conv_factor = NaN; 
else      % ratio between number of time steps, or total discharges, with above-threshold precipitation to total.
    conv_factor = nansum(orig_wt((P >= p_threshold) & ptfilter)) / sumwt; 
end


% This factor converts new water fractions to 'forward' new water fractions (equations 21 and 28).
if ~vol_wtd                   % if we are not volume-weighting
    if sum(isnan(Qfilt)) > 0    
        fwd_factor = NaN;               % if some Q's are NaN, we can't calculate PFnew
    else
        fwd_factor = nansum(Q((P >= p_threshold) & ptfilter)) / nansum(P((P >= p_threshold) & ptfilter)); % (equation 21)
    end
else                          % if we ARE volume-weighting
    fwd_factor = nansum(Q((P >= p_threshold) & ptfilter)) / nansum(P(ptfilter)); % (equation 28)
end
    
    
% return arguments
EHSoutput.QpFnew = beta_hat;                         %new water fraction for time steps with precipitation
EHSoutput.QpFnew_se = se;                                    %standard error
EHSoutput.QFnew = beta_hat * conv_factor ;                %new water fraction or all time steps, including rainless periods
EHSoutput.QFnew_se = se * conv_factor;                    %standard error
EHSoutput.PFnew = beta_hat * fwd_factor ;      %forward new water fraction (fraction of precipitation that becomes same-day streamflow)
EHSoutput.PFnew_se = se * fwd_factor;            %standard error


end

%%%%%%%%%%%%%%%%%%
% other subfunctions called by EHS_Fnew

function [x] = outlier_filter(x)
% filters outliers
% takes as input:
% x     vector of doubles, potentially with NaN values
% returns x with outliers replaced by NaN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

medx = nanmedian(x); % find the median of x
absdev = abs(x-medx); % absolute value of deviation of x's from medx
MAD = nanmedian(absdev); % median absolute deviation  
x(absdev>6*MAD) = NaN; % set x values to NaN if they deviate from median by more than 6*MAD

% Note that other exclusion criteria besides 6*MAD could be used.  A smaller
% multiplier on MAD (say, 3*MAD) will raise the breakdown point (the
% algorithm will be able to cope with larger numbers of outliers) but will
% also tend to exclude more of the real values and thus will make robust
% estimation less faithful to the patterns in the real data (depending, of
% course, on how far the tails of the real data extend).

end



function [wtd_covar] = wtd_cov(x,y,wt)
% calculate weighted covariance
% takes as input:
% x     vector of doubles, potentially with NaN values
% y     vector of doubles, potentially with NaN values
% wt    vector of weights, potentially with NaN values
% returns weighted covariance of x and y
% for weighted variance, do wtd_cov(x, x, wt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = length(x);
if isempty(y)       % then this is the correlation for x and x
    y = x;
end
  
%check input length
if ((length(y) ~= n) || (length(wt) ~=n)) 
    error('Fatal error: x, y, and wt must have same length in wtd_cov!')
end
  
%mask weight values for missing x and y
%this will have the effect that further calculations will be done on consistent sets of y and x
wt(isnan(x) | isnan(y)) = nan;
x(isnan(wt)) = nan;    
y(isnan(wt)) = nan;   

if (sum( ~isnan(wt) ) < 3) 
    error('Fatal error: need at least 3 non-missing pairs of x and y in wtd_cov!')
end
  
sumwt = nansum(wt);   % sum of weights
sumsq = nansum(wt.*wt);   % sum of squared weights

n_eff = sumwt.*sumwt / sumsq;       % effective sample size
wtd_avg_x = nansum(x.*wt)./nansum(wt);
wtd_avg_y = nansum(y.*wt)./nansum(wt);

% weighted covariance (equation 56 or 57 -- note that due to a typo there is a 
% factor of Q_j missing in the last summation in both eqs. 56 and 57 in K2019)
wtd_covar = (n_eff/(n_eff-1)) * nansum( wt.*(y - wtd_avg_y).*(x - wtd_avg_x))/sumwt;
% n.eff/(n.eff-1) is the correction for degrees of freedom

% note that this is equivalent to
% Galassi, Davies, Theiler, Gough, Jungman, Booth, and Rossi, GNU Scientific Library - Reference manual, Version 1.15, 2011. Sec. 21.7 Weighted Samples
% https://www.gnu.org/software/gsl/manual/html_node/Weighted-Samples.html%Weighted-Samples, accessed 05 Sept. 2018.

end 


function [wtd_correl] = wtd_cor(x, y, wt)
% calculate weighted correlation
% takes as input:
% x     vector of doubles, potentially with NaN values
% y     vector of doubles, potentially with NaN values
% wt    vector of weights, potentially with NaN values
% returns weighted correlation of x and y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
n = length(x);

%check input length
if ((length(y) ~= n) || (length(wt) ~=n)) 
    error('Fatal error: x, y, and wt must have same length in wtd_cor!\n')
end

%mask weight values for missing x and y
%this will have the effect that further calculations will be done on consistent sets of y and x
wt(isnan(x) | isnan(y)) = nan;
x(isnan(wt)) = nan;    
y(isnan(wt)) = nan;

if (sum( ~isnan(wt) ) < 3) 
  error('Fatal error: need at least 3 non-missing pairs of x and y in wtd_cor!\n')
end

wtd_avg_x = nansum(x.*wt)./nansum(wt);
wtd_avg_y = nansum(y.*wt)./nansum(wt);

% weighted correlation
wtd_correl = nansum(wt.*(y - wtd_avg_y).*(x - wtd_avg_x))./nansum(wt);        % wtd covariance without degrees-of-freedom correction
wtd_correl = wtd_correl / sqrt(nansum(wt.*(x - wtd_avg_x).*(x - wtd_avg_x))/nansum(wt));   %divide by weighted standard deviation
wtd_correl = wtd_correl / sqrt(nansum(wt.*(y - wtd_avg_y).*(y - wtd_avg_y))/nansum(wt));  %divide by weighted standard deviation

end 