function TTDoutput = EHS_TTD(inputdat, opts, TTDopts)
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% EHS_TTD    estimates transit time distribution by ensemble hydrograph separation, 
%%% with filtering and volume weighting
%
% Version 1.3  build 2020.06.21
% Author: James Kirchner and Julia Knapp, ETH Zurich
% This code was translated from R to MATLAB by Julia Knapp and tested for
% MATLAB R2018b and R2019b
%
% Copyright (C) 2020 ETH Zurich and James Kirchner
% Public use of this script is permitted under GNU General Public License 3 (GPL3); for details see <https://www.gnu.org/licenses/>
% 
%
% READ THIS CAREFULLY:
% ETH Zurich, James Kirchner and Julia Knapp make ABSOLUTELY NO WARRANTIES OF ANY KIND, including NO WARRANTIES, expressed or implied, that this software is
%    free of errors or is suitable for any particular purpose.  Users are solely responsible for determining the suitability and
%    reliability of this software for their own purposes.
%
% ALSO READ THIS:
% These scripts implement the ensemble hydrograph separation approach as presented in J.W. Kirchner, "Quantifying new water fractions
%    and transit time distributions using ensemble hydrograph separation: theory and benchmark tests", Hydrology and Earth System Sciences, 
%    23, 303-349, 2019.  These scripts are further described in J.W. Kirchner and J.L.A. Knapp, "Matlab and R scripts for ensemble hydrograph
%    separation", Hydrology and Earth System Sciences.  
%    Users should cite both of these papers (the first for the method, and the second for the scripts)


% The equation numbers in the scripts refer to the corresponding equations in Kirchner (2019).  The equations may differ in detail.  For example,
% in these scripts, k ranges from 1 to m+1 instead of 0 to m due to the array indexing conventions of the R language.
% Also, as described below, in calculating TTD's the counts of above-threshold precipitation n.xk and n.xkxl have been
% replaced by counts of usable inputs, denoted u.xk and u.xkxl.  Usable inputs either have precipitation tracer measurements,
% or have below-threshold precipitation so that one can assume that the tracer input was negligible.  The equations are adjusted accordingly.

% estimate transit time distributions, with or without volume-weighting
%    
% takes as input:
% Cp           vector of tracer concentration in precipitation (at equally spaced points in time)
% Cq           vector of tracer concentration in streamflow (assumed to be a vector at the same time points at Cp)
% P            vector of precipitation rate (over the same time intervals as Cp)
% Q            optional vector of discharges (in same units as P)
% p_threshold  threshold precipitation rate below which tracer inputs are ignored
% m            maximum lag in TTD (in number of time steps)
% vol_wtd      logical flag indicating whether volume-weighted TTD's are desired (TRUE) or not (FALSE)
% Pfilter      optional vector of flags (1 and 0, T and F, or TRUE and FALSE) indicating whether individual precipitation time steps should be included in the analysis
% Qfilter      optional vector of flags (1 and 0, T and F, or TRUE and FALSE) indicating whether individual discharge time steps should be included in the analysis
% robust       if TRUE, TTD estimation employs outlier exclusion and iteratively reweighted least squares
% ser_corr     if TRUE, standard error estimates account for serial correlation in residuals
%
% returns a list as output, with the following objects (each is a vector of length m+1)
% TTD        transit time distribution (if vol_wtd==TRUE, this TTD is weighted by discharge)
% se         standard error of TTD
% fwd_TTD    forward transit time distribution (if vol_wtd==TRUE, this TTD is weighted by precipitation)
% fwd_TTD_se standard error of forward TTD
%
% Cp and Cq can have missing values (NaN)
% P must not have missing values.  The same is true for Q, if it is present.
% Q  can be omitted, but if so, forward transit time distribution will be NaN, and *all* flow-weighted TTD's will be NaN
% Values of P (and Q, if present) must be non-negative
%
%
% IMPORTANT NOTE: This calculation proceeds differently from the one presented in K2019 in several key respects.
%
% Specifically, instead of excluding x values for missing concentrations (and those corresponding to below-threshold precipitation
% amounts) from the calculations of covariances, this calculation replaces them by zeroes (after the weighted mean is subtracted from
% each x variable).  This means that the missing values are no longer excluded, but instead are replaced by points that have no 
% leverage (since the variables are all centered around zero).  This simplifies the overall calculation scheme somewhat, and also
% allows explicit calculation of residuals and residual variance (for estimating parameter uncertainties).  The overall approach
% remains the same, but the missing values must be taken into account differently, since replacing a missing value with zero dilutes the
% calculated covariances (which is correct for below-threshold rainfall, but not for missing measurements), whereas excluding a missing
% value inflates the covariances (in the case of below-threshold rainfall), but doesn't inflate the covariances for missing measurements.
%
% In practice this means that the calculations proceed similarly, but n.xk and n.xkxl must be defined differently and must be handled
% differently.  In K2019, n_xk and n_xkxl are the numbers of x's that have above-threshold precipitation (and therefore should
% be part of the covariance calculation).  Here, instead, we count the number (or weight) of 'usable' x(:,k) values or x(:,k)x(:,l) pairs, 
% and denote these u_xk and u_xkxl (note the change in notation), where u stands for 'number usable'.  For an x(j,k) value to be 'usable',
% either P must be below the threshold (in which case it doesn't matter whether Cp is missing or not, because it shouldn't contribute 
% significantly to the covariances), or Cp(i=j-k) and C_ref(j) must both exist (so that the contribution to the covariance can be calculated).
% 
%
% Note also that the sampling interval delta_t is not explicitly represented here, as it is in Eqs. (55), (60), (63), and (66) of K2019.
% This means that the TTD's should be interpreted as a fraction of discharge (or precipitation) per sampling interval (i.e., per day if the sampling
% interval is one day, per week if the sampling interval is one week, etc.)
%
% The average time lag between precipitation and streamflow for each lag index k will be (k-0.5)*delta_t (for grab sampling of streamflow)
% or (k-0.75)*delta_t (for time-averaged sampling of streamflow).  These formulas differ from those in Sect. 4.1 of K2019 because
% here, the index k starts at 1 rather than 0.
%   
% In addition, filtering by precipitation time steps proceeds fundamentally differently than in K2019.  
% If any Pfilter==FALSE (filtering by precipitation time), we divide the input time series into
% two subsets (for Pfilter true and false), and analyze them jointly.  
% The subsets must be analyzed jointly, each with its own TTD coefficients, because
% the lagged precipitation inputs from each subset must be interleaved (since the "excluded"
% points will still affect the streamflow concentrations, and their effects must be accounted for in the 
% regression model (just not with the same coefficients).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin < 2       % create opts strcutrue
    opts = [];
end
if nargin < 3   % create TTDopts strcutrue
    TTDopts = [];
end

% extract data from structures
Cp = inputdat.Cp;
Cq = inputdat.Cq;
P = inputdat.p;



nu = 0.5;       % fractional weight given to Tikhonov-Phillips regularization

%%%%%%%%%%
% set default values
% inputdat
if isfield(inputdat,'q')   Q = inputdat.q;
else     Q = nan(size(Cp));     end

% opts
if isfield(opts,'ser_corr')     ser_corr = opts.ser_corr;
else     ser_corr = 1; end

if isfield(opts,'robust')     robust = opts.robust;
else     robust = 1;  end

if isfield(opts,'vol_wtd')     vol_wtd = opts.vol_wtd;
else     vol_wtd = 0;   end

if isfield(opts,'p_thresh')     p_threshold = opts.p_thresh;
else     p_threshold = 0;   end

% TTD opts
if isfield(TTDopts,'Qfilter')     Qfilter = TTDopts.Qfilter;
else    Qfilter = ones(size(P));    end

if isfield(TTDopts,'Pfilter')     Pfilter = TTDopts.Pfilter;
else     Pfilter = ones(size(P));   end

if isfield(TTDopts,'m')     m = TTDopts.m;
else     m = 10;   end



n = length(Cp); % n is total number of rows in our problem

% check input data
if ((length(Cq) ~= n) || (length(P) ~= n) || (length(Q) ~= n) || (length(Pfilter) ~= n) || (length(Qfilter) ~= n)) 
    error('Fatal error in EHS_TTD: Cq, Cp, P, Q, Pfilter and Qfilter must have same length!')
end

if sum(isnan(Pfilter)>0) 
    error('Fatal error in EHS_TTD: missing values in Pfilter!')
end
if sum(isnan(Qfilter)>0) 
    error('Fatal error in EHS_TTD: missing values in Qfilter!')
end

        
if (min(P) < 0)
    error('Fatal error in EHS_TTD: P must be >= 0!')
end
if (sum(isnan(P)) > 0)
    error('Fatal error in EHS_TTD: missing values in P!')
end
if (sum(~isnan(Q)) > 0) 
    if (min(Q) < 0)
        error('Fatal error in EHS_TTD: Q must be >=0!')
    end
    if (sum(isnan(Q)) > 0)
        error('Fatal error in EHS_TTD: missing values in Q!')
    end
end
    
% if we will be volume-weighting, then Q is the weight vector, otherwise just a set of 1's
if vol_wtd  
    wt = Q/nanmean(Q); % normalize weights to mean of 1 (this is just good numerical hygiene)
else 
    wt = ones(n,1);
end

% for robust estimation, first exclude extreme values of Cp and Cq (since they could potentially create  
% x's with leverage that is too strong for robust estimation to overcome)
if robust
    Cp = outlier_filter(Cp);
    Cq = outlier_filter(Cq);
end

% coerce Pfilter and Qfilter to logical (in case they were supplied as numeric vectors)
Pfilter = logical(Pfilter);
Qfilter = logical(Qfilter);
    
% replace Cp with NaN for P less than p_threshold
Cp(P < p_threshold) = nan;

% create Cq.ref (reference value of Cq, lagged one more than maximum TTD lag)
Cq_ref = [nan(m+1,1);Cq(1:end-1-m)];     % equation 34
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%% calculate y

% create y from Cq and Cq.ref
y = Cq - Cq_ref;         % equation 35

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% calculating x matrix is special!

if sum((~Pfilter)>0) 
    filterP = logical(1);
else
    filterP = logical(0);     % a flag to indicate whether we will be filtering by precipitation time or not
end
if filterP
    n_sets = 2;    
else
    n_sets = 1;     % if we are filtering by precipitation time, we have two sets of inputs
end
% we need to account for: those included by the filter, and those excluded by it
mm = n_sets*(m+1);            % mm is the width of the x matrix (and the total number of fitted coefficients)
  
x = nan(n , (mm));   % initialize the x matrix (double-wide if we are filtering by precipitation time)
if ~filterP         % if we are not filtering by precipitation
    for k = 1:(m + 1)
        x(:,k) = [nan(k-1,1);Cp(1:end - (k - 1))] - Cq_ref ;        % equation 35
    end
else
    z = nan(n,1);
    for k = 1:m+1
        lagPfilter = [nan(k-1,1);Pfilter(1:end - (k - 1))];
        xx = [nan(k-1,1);Cp(1:end - (k - 1))] - Cq_ref;
        x(:,k) = z;
        x(lagPfilter==1,k) = xx(lagPfilter==1);   % "in set" precipitation values (with NA's in-filled for out-of-set values)
        x(:,m+1+k) = z;
        x(lagPfilter==0,m+1+k) = xx(lagPfilter==0);  % # "out of set" precipitation values (with NA's in-filled for in-set values)
    end
    % Note that we must in-fill with NA's, not zeroes, because these are missing values
end                                                            % (not known concentrations of zero -- *flux* is zero but concentration is NA).
% These will eventually be replaced with zeroes after centering the known values.
   
% create matrix of boolean flags indicating whether each x value corresponds to above-threshold precipitation or not (whether or not Cp is missing)
above_thresh = nan(n,n_sets*(m+1));     % initialize the above threshold matrix (double-wide if we are filtering by precipitation time)

if ~filterP
    for k = 1:(m + 1)
        above_thresh(:,k) =  [nan(k-1,1);P(1:end-(k-1))]>= p_threshold;  
    end
else
    z = nan(n,1);
    for k = 1:m+1
        lagPfilter = [nan(k-1,1);Pfilter(1:end - (k - 1))];
        xx = [nan(k-1,1);P(1:end - (k - 1))]>= p_threshold;  
        above_thresh(:,k) = z;
        above_thresh(lagPfilter==1,k) = xx(lagPfilter==1);   % "in set" values (with FALSE in-filled for out-of-set values)
        above_thresh(:,m+1+k) = z;
        above_thresh(lagPfilter==0,m+1+k) = xx(lagPfilter==0);  % "out of set" values (with FALSE in-filled for in-set values)
    end
end
above_thresh(isnan(above_thresh)) = 0;
above_thresh = logical(above_thresh);

% create matrix of boolean flags indicating usable x values (where Cp and C.ref both exist, or P is below threshold and we can safely set x to zero below) 
usable = nan(n,n_sets*(m+1));     % initialize the "usable" matrix (double-wide if we are filtering by precipitation time)
for k = 1:(m + 1)
  usable(:,k) =  ( ~isnan(x(:,k)) | ~above_thresh(:,k));        % equation A6 of KK2020
  if filterP
      for k = m+1:mm
          usable(:,k) =  ( ~isnan(x(:,k)) | ~above_thresh(:,k));        % equation A6 of KK2020
      end
  end
end
usable = logical(usable);

% from here on, we will work with x and y, rather than with the original Cp and Cq


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% remove missing rows, center columns on weighted means, and in-fill individual missing x's with zeroes (so that they will have no leverage)

% here we apply Qfilter by setting y to NA (this will trigger the deletion of the entire row)
y(~Qfilter) = NaN;
        
% set y to NaN if all x's are NaN (since we won't be able to do anything with this row anyhow)
for j = 1:n 
    if sum(~isnan(x(j,:))) == 0
        y(j) = NaN;
    end
end
% delete rows of x where y is missing (since we won't be able to do anything with these anyhow)
x = x(~isnan(y),:);

% delete rows of 'usable' where y is missing (since we won't be able to do anything with these anyhow)
usable = usable(~isnan(y),:);

% also delete weights where y is missing
ww = wt(~isnan(y));         % note name change to ww

% and delete missing elements of y (otherwise the rows of y won't match the rows of x...!)
y = y(~isnan(y));

% subtract weighted mean from y's 
y = y - nansum(ww.*y)/nansum(ww(~isnan(y)));

% subtract weighted means from x's
meanx = nan(1,size(x,2));
for k = 1:size(x,2)
    meanx(k) = nansum(x(:,k).*ww)/nansum(ww(~isnan(x(:,k))));       % calculate column weighted means
end
x = x - repmat(meanx,size(x,1),1); % subtract the column means from each row of x

% replace missing values of x with zeroes (remember that these should have little leverage because we have already centered each of the x's)
x(isnan(x)) = 0;
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% optionally, calculate robustness weights

if robust
  [robustfit,rob_wt] = IRLS(y, x, ww);       % iteratively reweighted least squares -- we don't use this to fit the TTD, but just to 
                                             % downweight any rows with outliers
  ww = ww.*rob_wt;                           % multiply by robustness weights (which will be small for outliers)
end 


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set up weighted linear system

% create weighted covariance matrix
covar = wtd_covar_matrix(x, ww)  ;           % equation A7 of KK2020
    
% create weighted xy covariance vector
xy = nan(mm,1);
for k = 1:mm                                % equation A8 of KK2020
    xy(k) = wtd_cov(x(:,k),y,ww);
end

% now we need to weight the elements of the matrix by u_xkxl/u_xk 
% (analogous to Appendix B of the paper, but with modifications as described below)
% These are defined in equation A10 of KK2020

% calculate u.xk as vector of total count (or total weight) of x's that have usable P values (analogous to equation 45, but with u rather than n)
% note that 'usable' means that P is below threshold or Cp and Cq_ref are both not is missing
u_xk = nan(mm,1); % start with a vector of NaN
for k = 1:(mm) 
    u_xk(k) = nansum(ww(usable(:,k)));      % fill with sum of weights for x's that are not unusable
                                            % usable means that any x of zero should contribute zero to covariance: P is below threshold or Cp and Cq.ref are not missing
end

% calculate u_xkxl as matrix of total count (or total weight) with non-missing values for each lag in both xk and xl (analogous to equation 45, but with u rather than n)
u_xkxl = nan(mm,mm);
for k = 1:mm
    for l = 1:mm
       u_xkxl(k,l) = nansum(ww(usable(:,k) & usable(:,l)));  % fill with sum of weights that are usable in both x_k and x_l
                           % usable means that any x of zero should contribute zero to covariance: P is below threshold or Cp is not missing
    end
end


if (sum(isnan(u_xkxl)) > 0)
  error('problem with u_xkxl')
end
if (sum(isnan(u_xk)) > 0)
  error('problem with u_xk')
end
if (sum(isnan(covar)) > 0)
  error('problem with C matrix')
end

% Now adjust covariance matrix by both u_xk and u_xkxl.
% NOTE, however, that we need to do this BACKWARDS compared to the way it is done in K2019, because in this implementation
% the missing values are replaced by zeroes, and thus dilute the covariances (they contribute nothing to the cross-products, but their
% number or weight is counted in the denominator).  This is correct if the missing values correspond to below-threshold precipitation
% (those *should* dilute the covariances), but if precipitation is above the threshold and the concentration is missing for other
% reasons, we need to correct for the dilution of the covariance.  

% To correct for this dilution we we need to re-scale each element of 
% the covariance matrix 'covar' by the ratio between the total number (or weight) and u.xkxl, and we need to re-scale each element of
% the covariance array xy by the total number (or weight) and u_xk.  Equivalently, we can just re-scale each element of the covariance 
% array covar by u_xk/u_xkxl (the reciprocal of what was used in K2019, where all missing values were excluded from
% the covariance calculations, rather than being replaced with zeroes).
    
 
C = covar.* u_xk./u_xkxl;   % element-wise multiplication and division, not matrix operations! (equation A11 of KK2020)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% now we solve the regularized (and weighted) linear regression problem

% create regularization matrix
H = tikh(m + 1);
% If we are filtering by precip time, we can't use one big regularization matrix for the whole problem, 
% because it will lead to leakage between the end of one TTD and the start of another.
% Here instead we use a segmented regularization matrix, with two
% Tikhonov-Phillips matrices along the diagonal.  This isolates the regularization
% effects within each block of (m+1) coefficients.
if (filterP) 
    toprow = [H,zeros(size(H))];
    bottomrow = [zeros(size(H)),H];
    H = [toprow;bottomrow];
end
lambda = nu / (1.0 - nu) * sum(diag(C)) / sum(diag(H));              % (equation 50)
 

% now solve (regularized) linear system to get TTD coefficients      % (equation 46, and equation A12 of KK2020)
beta_hat = (C + (lambda * H))\ xy;
    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate standard errors

% Here we do not use Glasser's error variance estimate (equation 52) because it is not particularly reliable.
% Instead we calculate the error variance directly from residuals of y for each x.
% Because we have replaced the missing x's with zeroes, we can still estimate y (for which we need values for each x) without giving the 
% missing points too much influence (this works because the x's have already been 'centered' by subtracting their weighted mean).
        
e = y - x * beta_hat;  % these are the residuals (equation A13 of KK2020)
                       % (note that means have already been subtracted from y and the x's, so there is no intercept)

nn = length(e);        % number of rows in our problem (note that this is not n, the number of original data points, because some have been removed)

s2e = (nn-1)./(nn-mm-1).*wtd_cov(e,e,ww); % weighted variance of residuals, corrected for degrees of freedom (equation A14 of KK2020)
                                             % note wtd_cov already corrects for uneven weighting
                                             
% first we need to calculate the effective sample size 
% (note that we do *not* use Eq. 59 of K2019 because uneven weighting is already taken into account in s2e above)
n_eff = ones(mm,1);
for k = 1:mm
    n_eff(k) =nansum(usable(:,k)) ;     % count usable points (equation A17 of KK2020)
end
                                             
if ser_corr % adjust effective degrees of freedom for serial correlation in residuals
    %calculate the serial correlation coefficient (with geometric mean weights)
    r_sc = wtd_cor( e, [NaN;e(1:end-1)], sqrt(ww.*[NaN;ww(1:end-1)])); 
      
    %effective sample size, accounting for serial correlation in residuals
    n_eff = n_eff * (1.0 - r_sc) / (1.0 + r_sc);  % equation A16 of KK2020
end
    
% now we need to estimate the inverse of the regularized covariance matrix 
regC_inverse = inv((C + (lambda * H))); % inverse of regularized C matrix

% now calculate the parameter covariance matrix
beta_covar_matrix = (s2e./ n_eff).* (regC_inverse * C * regC_inverse); % covariance matrix of coefficients
% note that n_eff-m-1 is degrees of freedom (degrees of freedom were handled differently in Kirchner 2019)

% standard error is the square root of the diagonal of this matrix (equation 54)
% unless degrees of freedom are zero or negative, in which case assign NaN
dd = diag(beta_covar_matrix);  % diagonal of the matrix
se = nan(mm,1);   % initialize standard error as NaN
for k = 1:mm
    if dd(k)>0
        se(k) = sqrt(dd(k));
    end
end
% we can't do this with ifelse, because that will trigger a warning if any of the dd are negative


%%%%%%%%%%%
% calculate conversion factor
    
% we need this conversion factor, conv_factor, to account for rainless periods (equations 55 and 60)
% note that this uses wt rather than ww, which has been altered by deletion of rows from our problem
conv_factor = nan(mm,1); 
for k = 1:m+1       % here we have to account for the possibility that there has been filtering by both P and Q...!
    conv_factor(k) = nansum(wt(above_thresh(:,k) & Qfilter & logical([zeros(k-1,1);Pfilter(1:end-(k-1))])))./...
        nansum(wt(Qfilter & logical([zeros(k-1,1);Pfilter(1:end-(k-1))])));
    if filterP  % if we are filtering by precipitation time steps
        for k = 1:m+1   % note that here we need to reverse Pfilter (because we are counting the "out of set" values)
            conv_factor(k+m+1) = nansum(wt(above_thresh(:,k+m+1) & Qfilter & logical([zeros(k-1,1);~Pfilter(1:end-(k-1))])))./...
                nansum(wt(Qfilter & logical([zeros(k-1,1);~Pfilter(1:end-(k-1))])));
        end
    end
end
    
% we need this conversion facto, fwd_factor, to convert beta_hat to forward TTD (equation 63, 64, 65, 66)
% note that here, again, we have to account for the possibility that there has been filtering by both P and Q...!
fwd_factor = nan(mm,1);

if ~vol_wtd % not volume-weighted: eqs 63, 64, 65
    for k = 1:m+1
        fwd_factor(k) = nanmean(Q(above_thresh(:,k) & Qfilter & logical([zeros(k-1,1);Pfilter(1:end-(k-1))])))./...
            nanmean(P(above_thresh(:,1) & logical([Qfilter(k:end);zeros(k-1,1)]) & Pfilter));
    end
    if filterP  % if we are filtering by precipitation time steps
        for k = 1:m+1
            fwd_factor(k+m+1) = nanmean(Q(above_thresh(:,k+m+1) & Qfilter & logical([zeros(k-1,1);~Pfilter(1:end-(k-1))])))./...
                nanmean(P(above_thresh(:,m+2) & logical([Qfilter(k:end);zeros(k-1,1)]) & ~Pfilter));
        end
    end
else    % volume-weighted: eq 66 (with conv_factor included)
    for k = 1:m+1
        fwd_factor(k) = conv_factor(k) * nanmean(Q(Qfilter & logical([zeros(k-1,1);Pfilter(1:end-(k-1))])))./...
            nanmean(P(logical([Qfilter(k:end);zeros(k-1,1)]) & Pfilter));
    end
    if filterP  % if we are filtering by precipitation time steps
        for k = 1:m+1
        fwd_factor(k+m+1) = conv_factor(k+m+1) * nanmean(Q(Qfilter & logical([zeros(k-1,1);~Pfilter(1:end-(k-1))])))./...
            nanmean(P(logical([Qfilter(k:end);zeros(k-1,1)]) & ~Pfilter));      
        end
    end
end
      
      
%% return results
TTD = beta_hat.* conv_factor;     % transit time distribution (corrected for rainless periods)
TTD_se = se.* conv_factor;        % standard error of TTD
fwd_TTD = beta_hat.* fwd_factor;  % forward TTD
fwd_TTD_se = se.* fwd_factor;     % standard error of forward TTD

      
TTDoutput.TTD = beta_hat(1:m+1).* conv_factor(1:m+1);     % transit time distribution (corrected for rainless periods)
TTDoutput.TTD_se = se(1:m+1).* conv_factor(1:m+1);        % standard error of TTD
TTDoutput.fwd_TTD = beta_hat(1:m+1).* fwd_factor(1:m+1);  % forward TTD
TTDoutput.fwd_TTD_se = se(1:m+1).* fwd_factor(1:m+1);     % standard error of forward TTD         
    
end % end EHS_TTD

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% other subfunctions needed for EHS_TTD

function [wtd_covar] = wtd_cov(x,y,wt)
% calculate weighted covariance
% takes as input:
% x     vector of doubles, potentially with NaN values
% y     vector of doubles, potentially with NaN values
% wt    vector of weights, potentially with NaN values
% returns weighted covariance of x and y
% for weighted variance, do wtd_cov(x, x, wt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = length(x);
if isempty(y)       % then this is the correlation for x and x
    y = x;
end
  
%check input length
if ((length(y) ~= n) || (length(wt) ~=n)) 
    error('Fatal error: x, y, and wt must have same length in wtd_cov!')
end
  
%mask weight values for missing x and y
%this will have the effect that further calculations will be done on consistent sets of y and x
wt(isnan(x) | isnan(y)) = nan;
x(isnan(wt)) = nan;    
y(isnan(wt)) = nan;   

if (sum( ~isnan(wt) ) < 3) 
    error('Fatal error: need at least 3 non-missing pairs of x and y in wtd_cov!')
end
  
sumwt = nansum(wt);   % sum of weights
sumsq = nansum(wt.*wt);   % sum of squared weights

n_eff = sumwt.*sumwt / sumsq;       % effective sample size
wtd_avg_x = nansum(x.*wt)./nansum(wt);
wtd_avg_y = nansum(y.*wt)./nansum(wt);

% weighted covariance (equation 56 or 57 -- note that due to a typo there is a 
% factor of Q_j missing in the last summation in both eqs. 56 and 57 in K2019)
wtd_covar = (n_eff/(n_eff-1)) * nansum( wt.*(y - wtd_avg_y).*(x - wtd_avg_x))/sumwt;
% n.eff/(n.eff-1) is the correction for degrees of freedom

% note that this is equivalent to
% Galassi, Davies, Theiler, Gough, Jungman, Booth, and Rossi, GNU Scientific Library - Reference manual, Version 1.15, 2011. Sec. 21.7 Weighted Samples
% https://www.gnu.org/software/gsl/manual/html_node/Weighted-Samples.html%Weighted-Samples, accessed 05 Sept. 2018.

end 


function [wtd_correl] = wtd_cor(x, y, wt)
% calculate weighted correlation
% takes as input:
% x     vector of doubles, potentially with NaN values
% y     vector of doubles, potentially with NaN values
% wt    vector of weights, potentially with NaN values
% returns weighted correlation of x and y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
n = length(x);

%check input length
if ((length(y) ~= n) || (length(wt) ~=n)) 
    error('Fatal error: x, y, and wt must have same length in wtd_cor!\n')
end

%mask weight values for missing x and y
%this will have the effect that further calculations will be done on consistent sets of y and x
wt(isnan(x) | isnan(y)) = nan;
x(isnan(wt)) = nan;    
y(isnan(wt)) = nan;

if (sum( ~isnan(wt) ) < 3) 
  error('Fatal error: need at least 3 non-missing pairs of x and y in wtd_cor!\n')
end

wtd_avg_x = nansum(x.*wt)./nansum(wt);
wtd_avg_y = nansum(y.*wt)./nansum(wt);

% weighted correlation
wtd_correl = nansum(wt.*(y - wtd_avg_y).*(x - wtd_avg_x))./nansum(wt);        % wtd covariance without degrees-of-freedom correction
wtd_correl = wtd_correl / sqrt(nansum(wt.*(x - wtd_avg_x).*(x - wtd_avg_x))/nansum(wt));   %divide by weighted standard deviation
wtd_correl = wtd_correl / sqrt(nansum(wt.*(y - wtd_avg_y).*(y - wtd_avg_y))/nansum(wt));  %divide by weighted standard deviation

end 
 

function [x] = outlier_filter(x)
% filters outliers
% takes as input:
% x     vector of doubles, potentially with NaN values
% returns x with outliers replaced by NaN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

medx = nanmedian(x); % find the median of x
absdev = abs(x-medx); % absolute value of deviation of x's from medx
MAD = nanmedian(absdev); % median absolute deviation  
x(absdev>6*MAD) = NaN; % set x values to NaN if they deviate from median by more than 6*MAD

% Note that other exclusion criteria besides 6*MAD could be used.  A smaller
% multiplier on MAD (say, 3*MAD) will raise the breakdown point (the
% algorithm will be able to cope with larger numbers of outliers) but will
% also tend to exclude more of the real values and thus will make robust
% estimation less faithful to the patterns in the real data (depending, of
% course, on how far the tails of the real data extend).

end




function [C] = wtd_covar_matrix(X, wt)
% calculate weighted covariance matrix
% takes as input:
% x     matrix of doubles, potentially with NaN values
% wt    vector of weights, potentially with NaN values
% 
% returns weighted covariance matrix, with each element equalling the covariance between the corresponding columns of X
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

[n,m] = size(X);

% check input length
if (length(wt) ~=n) 
    error('Fatal error: x and wt must have same length in wtd_covar_matrix!')
end

C = nan(m,m); % start with a blank matrix

for j = 1:m 
    for k = 1:m 
        if j>k 
            C(j,k)=C(k,j); % do the lower triangle by symmetry
        else
            C(j,k) = wtd_cov(X(:,j), X(:,k), wt);  % call the weighted covariance routine, passing the relevant columns of X
        end
    end
end
  
end 




function [x] = tikh(m) 
% creates m x m regularization matrix (equation 49)
% create regularization matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
x = zeros(m,m); % create matrix filled with zeroes
  
% put values on diagonal
x(1, 1) = 1;
x(m, m) = 1;
x(2, 2) = 5;
x(m - 1, m - 1) = 5;
for i = 3:(m - 2)
    x(i, i) = 6;
end
  
% put values in innermost off-diagonal rows
x(1, 2) = -2;
x(2, 1) = -2;
x(m, m - 1) = -2;
x(m - 1, m) = -2;
for i = 3:(m - 1)
    x(i, i - 1) = -4;
    x(i - 1, i) = -4;
end
  
% put values in outermost off-diagonal rows
for i = 3:m
    x(i, i - 2) = 1;
    x(i - 2, i) = 1;
end
  
end % end tikh
