function EHS_demo
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EHS demo -- demonstration script for EHS ensemble hydrograph separation code
%
% Version 1.3  build 2020.06.21
% Authors: James Kirchner and Julia Knapp, ETH Zurich
% This code was translated from R to MATLAB by Julia Knapp and tested for
% MATLAB R2018b and R2019b
%
% Copyright (C) 2020 ETH Zurich and James Kirchner
% Public use of this script is permitted under GNU General Public License 3 (GPL3); for details see <https://www.gnu.org/licenses/>
%
% READ THIS CAREFULLY:
% ETH Zurich, James Kirchner and Julia Knapp make ABSOLUTELY NO WARRANTIES OF ANY KIND, including NO WARRANTIES, expressed or implied, that this software is
%    free of errors or is suitable for any particular purpose.  Users are solely responsible for determining the suitability and
%    reliability of this software for their own purposes.
%
% ALSO READ THIS:
% These scripts implement the ensemble hydrograph separation approach as presented in J.W. Kirchner, "Quantifying new water fractions
%    and transit time distributions using ensemble hydrograph separation: theory and benchmark tests", Hydrology and Earth System Sciences, 
%    23, 303-349, 2019.  These scripts are further described in J.W. Kirchner and J.L.A. Knapp, "Matlab and R scripts for ensemble hydrograph
%    separation", Hydrology and Earth System Sciences.  
%    Users should cite both of these papers (the first for the method, and the second for the scripts)


clear all 
close all
clc


%%%%%%%%%% subfunctions
% please note that inputs are passed as structures - this differs from the
% R-version of the script:
%    Fnew = EHS_Fnew(input_data, opts, Fnew_options)
%    profile = EHS_profile(input_data,opts,  profile_options)
%    TTD = EHS_TTD(input_data, opts, TTD_options)
%%%%
% Input:
%   input_data: structure with concentrations and water fluxes
%   Fnew_options: structure with option inputs relevant to Fnew calculations
%   profile_options: structure with option inputs relevant to EHS profiles
%   TTD_options: structure with option inputs relevant to TTD calculations
%%%%
% Mandatory, optional, and default settings of input structures:
%   input_data.Cp                   % tracer concentration in precipitation (per mil)
%   input_data.Cq                   % tracer concentration in streamflow (per mil)
%   input_data.p                    % daily precipitation (mm/day)
%   input_data.q                    % daily streamflow (mm/day)
%%%%
% opts: general options relevant to Fnew, profile, and TTD
%   opts.p_threshold                % (optional) threshold precipitation rate below which tracer inputs are ignored, 
%                                   % default: 0 (mm/day)
%   opts.vol_wtd                    % (optional) logical flag indicating whether volume-weighted new water fractions are desired (TRUE) or not (FALSE), 
%                                   % default: FALSE (0)
%   opts.robust                     % (optional) if TRUE (1), Fnew is estimated by iteratively reweighted least squares (a robust estimation method)
%                                   % default: TRUE (1)
%   opts.ser_corr                   % (optional) logical flag indicating whether volume-weighted new water fractions are desired (TRUE) or not (FALSE), 
%                                   % default: FALSE (1)
%%%%
% Fnew_options: options relevant to EHS_Fnew only
%   Fnew_options.ptfilter           % (optional) optional vector of flags (1 and 0 = TRUE and FALSE) indicating whether individual time steps should be included in the analysis, 
%                                   % default: vector of ones (all true)
%%%%
% profile_options: options relevant to EHS_profile only
%   profile_options.crit            % (mandatory) vector containing criterion for subsetting (for example, Q or P)
%   profile_options.lwr             % (optional) lower bounds of subsets in percentile ranks
%                                   % default: [0,10,20,30,40,50,60,70,80,90,95]
%   profile_options.upr             % (optional) upper bounds of subsets in percentile ranks
%                                   % default: [20,30,40,50,60,70,80,90,100,100,100]
%   profile_options.ptfilter        % (optional) optional vector of flags (1 and 0 = TRUE and FALSE) indicating whether individual time steps should be included in the analysis, 
%                                   % default: vector of ones (all true)
%%%%
% TTD_options: options relevant to EHS_TTD only
%   TTD_options.m                   % (optional) maximum lag in TTD (in number of time steps), 
%                                   % default: 10
%   TTD_options.Pfilter             % (optional) optional vector of flags (1 and 0, T and F, or TRUE and FALSE) indicating whether individual precipitation time steps should be included in the analysis 
%                                   % default: vector of ones (all true)
%   TTD_options.Qfilter             % (optional) optional vector of flags (1 and 0, T and F, or TRUE and FALSE) indicating whether individual discharge time steps should be included in the analysis 
%                                   % default: vector of ones (all true)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



format long
%%%%%%%%%%%%%%%%%%%%%%%%
% supply input filename

input_filename = 'EHS demo daily input1.txt';     %replace filename as needed for other data sets

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% here we read the data file 
warning off
data = readtable(input_filename);    %this is the test data file
warning on
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% overview of 'daily demo input1.txt'
% .date              : Factor w/ 1826 levels '01-Apr-91','01-Apr-92',..:   % date label (not used)
% .year              : num  91 91 91 91 91 ...                             % decimal year (not used)
% .p                 : num  0.01 0 0 0 0 ...                               % daily precipitation (mm/day)
% .q                 : num  1.95 1.85 1.76 1.69 1.63 ...                   % daily streamflow (mm/day)
% .Cp                : num  NA NA NA NA NA ...                             % tracer concentration in precipitation (per mil)
% .Cq                : num  -46.3 NA -44.7 -46.4 -43.9 ...                 % tracer concentration in streamflow (per mil)
% .pTop20            : int  1 0 0 0 0 0 0 0 0 1 ...                        % flag for top 20% of precipitation volumes
% .qTop20            : int  1 0 1 0 1 0 0 1 1 1 ...                        % flag for top 20% of streamflow rates
% .Cpcorrupted1pct : num  -7.53 -7.46 -11.98 -9 -12.12 ...                 % tracer concentration in precipitation, with 1% of points replaced by outliers
% .Cqcorrupted1pct : num  -8.56 -8.13 -8.83 -8.78 -9.38 ...                % tracer concentration in streamflow, with 1% of points replaced by outliers
% .Cpcorrupted2pct : num  -7.53 -7.46 -11.98 -9 -12.12 ...                 % tracer concentration in precipitation, with 2% of points replaced by outliers
% .Cqcorrupted2pct : num  -8.56 -8.13 -8.83 -8.78 -9.38 ...                % tracer concentration in streamflow, with 2% of points replaced by outliers
% .Cpcorrupted5pct : num  -7.53 -7.46 -11.98 -9 -12.12 ...
% .Cqcorrupted5pct : num  -8.56 -8.13 -8.83 -8.78 -9.38 ...
% .Cpcorrupted10pct: num  -7.53 -7.46 -11.98 -9 -12.12 ...
% .Cqcorrupted10pct: num  -27.2 -8.13 -8.83 -8.78 -9.38 ...
% .Cpcorrupted20pct: num  -7.53 -7.46 -11.98 -40.7 -12.12 ...
% .Cqcorrupted20pct: num  -27.2 -8.13 -8.83 -8.78 -61.15 ...
% .Cpcorrupted30pct: num  -7.53 -7.46 -11.98 -40.7 -35.16 ...
% .Cqcorrupted30pct: num  -27.2 -8.13 -8.83 -25.78 -61.15 ...
% .Cpcorrupted40pct: num  -7.53 -7.46 -11.98 -40.7 -35.16 ...
% .Cqcorrupted40pct: num  -27.2 -8.13 -8.83 -25.78 -61.15 ...
% .Cpcorrupted50pct: num  -7.53 -7.46 -28.32 -40.7 -35.16 ...
% .Cqcorrupted50pct: num  -27.2 -8.13 -8.83 -25.78 -61.15 ...

% note that the date and year information is not used here
% instead, it is assumed that the rows of the input file represent an evenly spaced set of times

% now assign these four variables to their columns in the data table
input_data.p = data.p;      %precipitation water flux
input_data.q = data.q;      %streamflow water flux

Pfilter = data.pTop20;   % filter for top 20% of (above-threshold) precipitation values
Qfilter = data.qTop20;   % filter for top 20% of discharge values

% now set these options
p_thresh = 1;  % this is the threshold precipitation rate (in P units per time step) below which P tracer inputs will be ignored

nttd = 25;   % number of time steps for TTDs
TTD_options.m = nttd-1;  % maximum lag in TTD (in number of time steps)

% these are the lag times associated with each time step
TTD_lag = [1:nttd] - 0.5;  % here we treat Q and Cq as being instantaneously measured
% if instead they are time-averaged, then replace -0.5 with -0.75 
% (see Kirchner, 2019, but note that here the index starts at 1 rather than 0)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Note that the analyses below assume that the default options are
%   vol_wtd  = FALSE
%   robust   = TRUE
%   ser_corr = TRUE
%   p_thresh = 1;
% these may be different from the defaults of the default settings of the
% subfunctions!
default_opts.vol_wtd = 0;
default_opts.robust = 1;
default_opts.ser_corr = 1;
default_opts.p_thresh = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Analyses WITHOUT outliers 

% set label for output filenames
output_label = 'EHS demo daily input1 no outliers';  %label for output files

input_data.Cp = data.Cp;    %precipitation tracer concentration (or del value)
input_data.Cq = data.Cq;    %streamflow tracer concentration (or del value)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate transit time distributions (TTDs)
TTD = EHS_TTD(input_data,default_opts,TTD_options);     % only TTD_options.m defined

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate flow-weighted TTDs
opts = default_opts;
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data,opts,TTD_options); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate TTDs for top 20% of discharges
TTD_options.Qfilter = Qfilter;
qTop20TTD = EHS_TTD(input_data, default_opts,TTD_options);   % TTD_options.m and TTD_options.Qfilter defined

TTD_options = rmfield(TTD_options,'Qfilter');   % remove Qfilter option
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate TTDs for top 20% of precipitation
TTD_options.Pfilter = Pfilter;
pTop20TTD = EHS_TTD(input_data, default_opts,TTD_options);      % TTD_options.m and TTD_options.Pfilter defined

TTD_options = rmfield(TTD_options,'Pfilter');   % remove Pfilter option
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% write file of TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se',...
    'qTop20TTD_TTD', 'qTop20TTD_TTD_se', 'pTop20TTD_fwd_TTD', 'pTop20TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se,...
    qTop20TTD.TTD, qTop20TTD.TTD_se, pTop20TTD.fwd_TTD, pTop20TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD qTop20TTD pTop20TTD

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate transit time distributions (TTDs) WITHOUT ROBUST ESTIMATION
opts = default_opts;
opts.robust = 0;
TTD = EHS_TTD(input_data, opts, TTD_options);        % only TTD_options.m defined; non-robust

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate flow-weighted TTDs WIHTOUT ROBUST ESTIMATION
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);    % only TTD_options.m defined; vol-wtd, non-robust

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate TTDs for top 20% of discharges WITHOUT ROBUST ESTIMATION
opts.vol_wtd = 0;
TTD_options.Qfilter = Qfilter;
qTop20TTD = EHS_TTD(input_data, opts, TTD_options);  % TTD_options.m and TTD_options.Qfilter defined, non-robust

TTD_options = rmfield(TTD_options,'Qfilter');   % remove Qfilter option
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate TTDs for top 20% of precipitation WITHOUT ROBUST ESTIMATION
TTD_options.Pfilter = Pfilter;
pTop20TTD = EHS_TTD(input_data, opts, TTD_options);  % TTD_options.m and TTD_options.Pfilter defined, non-robust

TTD_options = rmfield(TTD_options,'Pfilter');   % remove Pfilter option

% write file of robust TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se',...
    'qTop20TTD_TTD', 'qTop20TTD_TTD_se', 'pTop20TTD_fwd_TTD', 'pTop20TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se,...
    qTop20TTD.TTD, qTop20TTD.TTD_se, pTop20TTD.fwd_TTD, pTop20TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD qTop20TTD pTop20TTD

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % estimate and write out profile of robust Fnew for different ranges of discharge
profile_options.crit = input_data.q;
profile = EHS_profile(input_data,default_opts,profile_options);
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust profile by Q ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate and write out profile of Fnew for different ranges of precipitation
profile_options.crit = input_data.p;
profile = EHS_profile(input_data,default_opts,profile_options);
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust profile by P ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % estimate and write out profile of non-robust Fnew for different ranges of discharge
opts = default_opts;
opts.robust = 0;
profile_options.crit = input_data.q;
profile = EHS_profile(input_data,opts,profile_options);
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust profile by Q ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate and write out profile of non-robust Fnew for different ranges of precipitation
profile_options.crit = input_data.p;
profile = EHS_profile(input_data,opts,profile_options);       % non-robust!
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust profile by P ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate (robust, unweighted) new water fractions and write them to the console
Fnew = EHS_Fnew(input_data, default_opts);
fprintf('\nEHS_Fnew(Cp, Cq, p, q, p_thresh):\n')
fprintf('QpFnew:    %1.9f\n',Fnew.QpFnew)
fprintf('QpFnew_se: %1.9f\n',Fnew.QpFnew_se)
fprintf('QFnew:     %1.9f\n',Fnew.QFnew)
fprintf('QFnew_se:  %1.9f\n',Fnew.QFnew_se)
fprintf('PFnew:     %1.9f\n',Fnew.PFnew)
fprintf('PFnew_se:  %1.9f\n\n',Fnew.PFnew_se)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate (non-robust, unweighted) new water fractions and write them to the console
opts = default_opts;
opts.robust = 0;
Fnew = EHS_Fnew(input_data, opts);
fprintf('\nEHS_Fnew(Cp, Cq, p, q, robust=0, p_thresh):\n')
fprintf('QpFnew:    %1.9f\n',Fnew.QpFnew)
fprintf('QpFnew_se: %1.9f\n',Fnew.QpFnew_se)
fprintf('QFnew:     %1.9f\n',Fnew.QFnew)
fprintf('QFnew_se:  %1.9f\n',Fnew.QFnew_se)
fprintf('PFnew:     %1.9f\n',Fnew.PFnew)
fprintf('PFnew_se:  %1.9f\n\n',Fnew.PFnew_se)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate (robust, weighted) new water fractions and write them to the console
opts = default_opts;
opts.vol_wtd = 1;
Fnew = EHS_Fnew(input_data, opts);
fprintf('\nEHS_Fnew(Cp, Cq, p, q, p_thresh, vol_wtd):\n')
fprintf('QpFnew:    %1.9f\n',Fnew.QpFnew)
fprintf('QpFnew_se: %1.9f\n',Fnew.QpFnew_se)
fprintf('QFnew:     %1.9f\n',Fnew.QFnew)
fprintf('QFnew_se:  %1.9f\n',Fnew.QFnew_se)
fprintf('PFnew:     %1.9f\n',Fnew.PFnew)
fprintf('PFnew_se:  %1.9f\n\n',Fnew.PFnew_se)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate (non-robust, weighted) new water fractions and write them to the console
opts = default_opts;
opts.vol_wtd = 1;
opts.robust = 0;
Fnew = EHS_Fnew(input_data, opts);
fprintf('\nEHS_Fnew(Cp, Cq, p, q, robust=0, p_thresh, vol_wtd):\n')
fprintf('QpFnew:    %1.9f\n',Fnew.QpFnew)
fprintf('QpFnew_se: %1.9f\n',Fnew.QpFnew_se)
fprintf('QFnew:     %1.9f\n',Fnew.QFnew)
fprintf('QFnew_se:  %1.9f\n',Fnew.QFnew_se)
fprintf('PFnew:     %1.9f\n',Fnew.PFnew)
fprintf('PFnew_se:  %1.9f\n\n',Fnew.PFnew_se)






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% analysis WITH 5 percent of points as outliers

% set label for output filenames
output_label = 'EHS demo daily input1 5pct outliers';  %label for output files

input_data.Cp = data.CpCorrupted5pct;    %precipitation tracer concentration (or del value)
input_data.Cq = data.CqCorrupted5pct;    %streamflow tracer concentration (or del value)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate transit time distributions (TTDs)
TTD = EHS_TTD(input_data, default_opts, TTD_options);

% estimate flow-weighted TTDs
opts = default_opts;
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);

% estimate TTDs for top 20% of discharges
TTD_options.Qfilter = Qfilter;
qTop20TTD = EHS_TTD(input_data, default_opts, TTD_options);

TTD_options = rmfield(TTD_options,'Qfilter');   % remove Qfilter option

% estimate TTDs for top 20% of precipitation
TTD_options.Pfilter = Pfilter;
pTop20TTD = EHS_TTD(input_data, default_opts, TTD_options);

TTD_options = rmfield(TTD_options,'Pfilter');   % remove Pfilter option

% write file of TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se',...
    'qTop20TTD_TTD', 'qTop20TTD_TTD_se', 'pTop20TTD_fwd_TTD', 'pTop20TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se,...
    qTop20TTD.TTD, qTop20TTD.TTD_se, pTop20TTD.fwd_TTD, pTop20TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD pTop20TTD qTop20TTD



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate transit time distributions (TTDs) WITHOUT ROBUST ESTIMATION
opts = default_opts;
opts.robust = 0;
TTD = EHS_TTD(input_data, opts, TTD_options);

% estimate flow-weighted TTDs WITHOUT ROBUST ESTIMATION
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);

% estimate TTDs for top 20% of discharges WITHOUT ROBUST ESTIMATION
opts = default_opts;
opts.robust = 0;
TTD_options.Qfilter = Qfilter;
qTop20TTD = EHS_TTD(input_data, opts, TTD_options);

TTD_options = rmfield(TTD_options,'Qfilter');   % remove Qfilter option
% estimate TTDs for top 20% of precipitation WITHOUT ROBUST ESTIMATION
TTD_options.Pfilter = Pfilter;
pTop20TTD = EHS_TTD(input_data, opts, TTD_options);

TTD_options = rmfield(TTD_options,'Pfilter');   % remove Pfilter option

% write file of non-robust TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se',...
    'qTop20TTD_TTD', 'qTop20TTD_TTD_se', 'pTop20TTD_fwd_TTD', 'pTop20TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se,...
    qTop20TTD.TTD, qTop20TTD.TTD_se, pTop20TTD.fwd_TTD, pTop20TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD pTop20TTD qTop20TTD



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate and write out profile of robust Fnew for different ranges of discharge
profile_options.crit = input_data.q;
profile = EHS_profile(input_data,default_opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust profile by Q ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out profile of robust Fnew for different ranges of precipitation
profile_options.crit = input_data.p;
profile = EHS_profile(input_data,default_opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust profile by P ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out profile of non-robust Fnew for different ranges of discharge
opts = default_opts;
opts.robust = 0;
profile_options.crit = input_data.q;
profile = EHS_profile(input_data,opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust profile by Q ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out profile of non-robust Fnew for different ranges of precipitation
profile_options.crit = input_data.p;
profile = EHS_profile(input_data,opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust profile by P ranges.txt'],'Delimiter','\t','WriteVariableNames',true)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate (robust, unweighted) new water fractions and write them to the console
Fnew = EHS_Fnew(input_data, default_opts);
fprintf('\nEHS_Fnew(Cp, Cq, p, q, p_thresh):\n')
fprintf('QpFnew:    %1.9f\n',Fnew.QpFnew)
fprintf('QpFnew_se: %1.9f\n',Fnew.QpFnew_se)
fprintf('QFnew:     %1.9f\n',Fnew.QFnew)
fprintf('QFnew_se:  %1.9f\n',Fnew.QFnew_se)
fprintf('PFnew:     %1.9f\n',Fnew.PFnew)
fprintf('PFnew_se:  %1.9f\n\n',Fnew.PFnew_se)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate (non-robust, unweighted) new water fractions and write them to the console
opts = default_opts;
opts.robust = 0;
Fnew = EHS_Fnew(input_data, opts);
fprintf('\nEHS_Fnew(Cp, Cq, p, q, robust=0, p_thresh):\n')
fprintf('QpFnew:    %1.9f\n',Fnew.QpFnew)
fprintf('QpFnew_se: %1.9f\n',Fnew.QpFnew_se)
fprintf('QFnew:     %1.9f\n',Fnew.QFnew)
fprintf('QFnew_se:  %1.9f\n',Fnew.QFnew_se)
fprintf('PFnew:     %1.9f\n',Fnew.PFnew)
fprintf('PFnew_se:  %1.9f\n\n',Fnew.PFnew_se)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate (robust, weighted) new water fractions and write them to the console
opts = default_opts;
opts.vol_wtd = 1;
Fnew = EHS_Fnew(input_data, opts);
fprintf('\nEHS_Fnew(Cp, Cq, p, q, p_thresh, vol_wtd):\n')
fprintf('QpFnew:    %1.9f\n',Fnew.QpFnew)
fprintf('QpFnew_se: %1.9f\n',Fnew.QpFnew_se)
fprintf('QFnew:     %1.9f\n',Fnew.QFnew)
fprintf('QFnew_se:  %1.9f\n',Fnew.QFnew_se)
fprintf('PFnew:     %1.9f\n',Fnew.PFnew)
fprintf('PFnew_se:  %1.9f\n\n',Fnew.PFnew_se)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate (non-robust, weighted) new water fractions and write them to the console
opts.robust = 0;
Fnew = EHS_Fnew(input_data, opts);
fprintf('\nEHS_Fnew(Cp, Cq, p, q, robust=0, p_thresh, vol_wtd):\n')
fprintf('QpFnew:    %1.9f\n',Fnew.QpFnew)
fprintf('QpFnew_se: %1.9f\n',Fnew.QpFnew_se)
fprintf('QFnew:     %1.9f\n',Fnew.QFnew)
fprintf('QFnew_se:  %1.9f\n',Fnew.QFnew_se)
fprintf('PFnew:     %1.9f\n',Fnew.PFnew)
fprintf('PFnew_se:  %1.9f\n\n',Fnew.PFnew_se)




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now let's look at daily input 2 (from a benchmark model that gives a humped transit time distribution)

%%%%%%%%%%%%%%%%%%%%%%%%
% supply input and output filenames

input_filename = 'EHS demo daily input2.txt';     %replace filename as needed for other data sets

output_label = 'EHS demo daily input2 no outliers';  %label for output files


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% here we read the data file 

warning off
data = readtable(input_filename);    %this is the test data file
warning on

input_data.p = data.p;      %precipitation water flux
input_data.q = data.q;      %streamflow water flux

Pfilter = data.pTop20;   % filter for top 20% of (above-threshold) precipitation values
Qfilter = data.qTop20;   % filter for top 20% of discharge values

input_data.Cp = data.Cp;
input_data.Cq = data.Cq;


% now set these options
default_opts.p_thresh = 1;  % this is the threshold precipitation rate (in P units) below which P tracer inputs will be ignored

nttd = 25;   % number of time steps for TTDs
TTD_options.m = nttd-1;  % maximum lag in TTD (in number of time steps)
% these are the lag times associated with each time step
TTD_lag = [1:nttd] - 0.5;  % here we treat Q and Cq as being instantaneously measured



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate transit time distributions (TTDs)
TTD = EHS_TTD(input_data, default_opts, TTD_options);

% estimate flow-weighted TTDs
opts = default_opts;
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);

% write file of TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD

% estimate transit time distributions (TTDs) WITHOUT ROBUST ESTIMATION
opts = default_opts;
opts.robust = 0;
TTD = EHS_TTD(input_data, opts, TTD_options);

% estimate flow-weighted TTDs WITHOUT ROBUST ESTIMATION
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);

% write file of TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate and write out profile of robust Fnew for different ranges of discharge
profile_options.crit = input_data.q;
profile = EHS_profile(input_data,default_opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust profile by Q ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out profile of robust Fnew for different ranges of precipitation
profile_options.crit = input_data.p;
profile = EHS_profile(input_data,default_opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust profile by P ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out non-robust profile of Fnew for different ranges of discharge
opts = default_opts;
opts.robust = 0;
profile_options.crit = input_data.q;
profile = EHS_profile(input_data,opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust profile by Q ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out non-robust profile of Fnew for different ranges of precipitation
profile_options.crit = input_data.p;
profile = EHS_profile(input_data,opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust profile by P ranges.txt'],'Delimiter','\t','WriteVariableNames',true)






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% now the same TTDs, but with 5 % outliers

% set label for output filenames
output_label = 'EHS demo daily input2 5pct outliers';  %label for output files

input_data.Cp = data.CpCorrupted5pct;    %precipitation tracer concentration (or del value)
input_data.Cq = data.CqCorrupted5pct;    %streamflow tracer concentration (or del value)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate transit time distributions (TTDs)
TTD = EHS_TTD(input_data, default_opts, TTD_options);

% estimate flow-weighted TTDs
opts = default_opts;
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);

% estimate TTDs for top 20% of discharges
TTD_options.Qfilter = Qfilter;
qTop20TTD = EHS_TTD(input_data, default_opts, TTD_options);

TTD_options = rmfield(TTD_options,'Qfilter');   % remove Qfilter option

% estimate TTDs for top 20% of precipitation
TTD_options.Pfilter = Pfilter;
pTop20TTD = EHS_TTD(input_data, default_opts, TTD_options);

TTD_options = rmfield(TTD_options,'Pfilter');   % remove Pfilter option


% write file of TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se',...
    'qTop20TTD_TTD', 'qTop20TTD_TTD_se', 'pTop20TTD_fwd_TTD', 'pTop20TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se,...
    qTop20TTD.TTD, qTop20TTD.TTD_se, pTop20TTD.fwd_TTD, pTop20TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD pTop20TTD qTop20TTD



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate transit time distributions (TTDs) WITHOUT ROBUST ESTIMATION
opts = default_opts;
opts.robust = 0;
TTD = EHS_TTD(input_data, opts, TTD_options);

% estimate flow-weighted TTDs WITHOUT ROBUST ESTIMATION
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);

% estimate TTDs for top 20% of discharges
opts = default_opts;
opts.robust = 0;
TTD_options.Qfilter = Qfilter;
qTop20TTD = EHS_TTD(input_data, opts, TTD_options);

TTD_options = rmfield(TTD_options,'Qfilter');   % remove Qfilter option

% estimate TTDs for top 20% of precipitation
TTD_options.Pfilter = Pfilter;
pTop20TTD = EHS_TTD(input_data, opts, TTD_options);

TTD_options = rmfield(TTD_options,'Pfilter');   % remove Pfilter option

% write file of non-robust TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se',...
    'qTop20TTD_TTD', 'qTop20TTD_TTD_se', 'pTop20TTD_fwd_TTD', 'pTop20TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se,...
    qTop20TTD.TTD, qTop20TTD.TTD_se, pTop20TTD.fwd_TTD, pTop20TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD pTop20TTD qTop20TTD




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now let's look at weekly input 1

%%%%%%%%%%%%%%%%%%%%%%%%
% supply input and output filenames

input_filename = 'EHS demo weekly input1.txt';     %replace filename as needed for other data sets

output_label = 'EHS demo weekly input1 no outliers';  %label for output files


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% here we read the data file 

warning off
data = readtable(input_filename);    %this is the test data file
warning on

input_data.p = data.p;      %precipitation water flux
input_data.q = data.q;      %streamflow water flux

input_data.Cp = data.Cp;
input_data.Cq = data.Cq;


% now set these options
default_opts.p_thresh = 1;  % this is the threshold precipitation rate (in P units) below which P tracer inputs will be ignored

nttd = 10;      % number of time steps for TTDs
TTD_options.m = nttd - 1;   % maximum lag in TTD (in number of time steps)

% these are the lag times associated with each time step
TTD_lag = [1:nttd] - 0.5;  % here we treat Q and Cq as being instantaneously measured



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate transit time distributions (TTDs)
TTD = EHS_TTD(input_data, default_opts, TTD_options);

% estimate flow-weighted TTDs
opts = default_opts;
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);

% write file of TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD


% estimate non-robust transit time distributions (TTDs)
opts = default_opts;
opts.robust = 0;
TTD = EHS_TTD(input_data, opts, TTD_options);

% estimate non-robust flow-weighted TTDs
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);

% write file of TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate and write out profile of Fnew for different ranges of discharge
profile_options.crit = input_data.q;
profile = EHS_profile(input_data,default_opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust profile by Q ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out non-robust profile of Fnew for different ranges of precipitation
profile_options.crit = input_data.p;
profile = EHS_profile(input_data,default_opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust profile by P ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out non-robust profile of Fnew for different ranges of discharge
opts = default_opts;
opts.robust = 0;
profile_options.crit = input_data.q;
profile = EHS_profile(input_data,opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust profile by Q ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out profile of Fnew for different ranges of precipitation
opts = default_opts;
opts.robust = 0;
profile_options.crit = input_data.p;
profile = EHS_profile(input_data,opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust profile by P ranges.txt'],'Delimiter','\t','WriteVariableNames',true)






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now let's look at weekly input 2 (from a benchmark model that gives a humped transit time distribution)

%%%%%%%%%%%%%%%%%%%%%%%%
% supply input and output filenames

input_filename = 'EHS demo weekly input2.txt';     %replace filename as needed for other data sets

output_label = 'EHS demo weekly input2 no outliers';  %label for output files


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% here we read the data file 

warning off
data = readtable(input_filename);    %this is the test data file
warning on

input_data.p = data.p;      %precipitation water flux
input_data.q = data.q;      %streamflow water flux

input_data.Cp = data.Cp;
input_data.Cq = data.Cq;


% now set these options
default_opts.p_thresh = 1;  % this is the threshold precipitation rate (in P units) below which P tracer inputs will be ignored

nttd = 10;    % number of time steps for TTDs
TTD_options.m = nttd - 1; % maximum lag in TTD (in number of time steps)
% these are the lag times associated with each time step
TTD_lag = [1:nttd] - 0.5;  % here we treat Q and Cq as being instantaneously measured



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate transit time distributions (TTDs)
TTD = EHS_TTD(input_data, default_opts, TTD_options);

% estimate flow-weighted TTDs
opts = default_opts;
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);

% write file of TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD

% estimate non-robust transit time distributions (TTDs)
opts = default_opts;
opts.robust = 0;
TTD = EHS_TTD(input_data, opts, TTD_options);

% estimate non-robust flow-weighted TTDs
opts.vol_wtd = 1;
wtd_TTD = EHS_TTD(input_data, opts, TTD_options);

% write file of TTDs
tableheader = {'TTD_lag','TTD_TTD', 'TTD_TTD_se', 'TTD_fwd_TTD','TTD_fwd_TTD_se',...
    'wtd_TTD_TTD', 'wtd_TTD_TTD_se', 'wtd_TTD_fwd_TTD', 'wtd_TTD_fwd_TTD_se'};
T = cell2table([num2cell([TTD_lag', TTD.TTD, TTD.TTD_se, TTD.fwd_TTD, TTD.fwd_TTD_se,...
    wtd_TTD.TTD, wtd_TTD.TTD_se, wtd_TTD.fwd_TTD, wtd_TTD.fwd_TTD_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust TTDs.txt'],'Delimiter','\t','WriteVariableNames',true)
clear TTD wtd_TTD


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate and write out profile of Fnew for different ranges of discharge
profile_options.crit = input_data.q;
profile = EHS_profile(input_data,default_opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust profile by Q ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out profile of Fnew for different ranges of precipitation
profile_options.crit = input_data.p;
profile = EHS_profile(input_data,default_opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' robust profile by P ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out non-robust profile of Fnew for different ranges of discharge
opts = default_opts;
opts.robust = 0;
profile_options.crit = input_data.q;
profile = EHS_profile(input_data,opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust profile by Q ranges.txt'],'Delimiter','\t','WriteVariableNames',true)

% estimate and write out profile of Fnew for different ranges of precipitation
opts = default_opts;
opts.robust = 0;
profile_options.crit = input_data.p;
profile = EHS_profile(input_data,opts,profile_options);       
tableheader = fieldnames(profile);
T = cell2table([num2cell([profile.lwr_limit,profile.upr_limit,profile.bin_avg,...
    profile.bin_avg_with_P,profile.QpFnew,profile.QpFnew_se,...
    profile.QFnew,profile.QFnew_se,profile.PFnew,profile.PFnew_se])],'VariableNames',tableheader);
writetable(T,[output_label,' non-robust profile by P ranges.txt'],'Delimiter','\t','WriteVariableNames',true)






end



