function EHSoutput = EHS_profile(inputdat, opts, profileopts)
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% EHS_profile    calculates profiles of Fnew for different percentile ranges of precipitation or discharge
%
% Version 1.3  build 2020.06.21
% Author: James Kirchner and Julia Knapp, ETH Zurich
% This code was translated from R to MATLAB by Julia Knapp and tested for
% MATLAB R2018b and R2019b
%
% Copyright (C) 2020 ETH Zurich and James Kirchner
% Public use of this script is permitted under GNU General Public License 3 (GPL3); for details see <https://www.gnu.org/licenses/>
% 
%
% READ THIS CAREFULLY:
% ETH Zurich, James Kirchner and Julia Knapp make ABSOLUTELY NO WARRANTIES OF ANY KIND, including NO WARRANTIES, expressed or implied, that this software is
%    free of errors or is suitable for any particular purpose.  Users are solely responsible for determining the suitability and
%    reliability of this software for their own purposes.
%
% ALSO READ THIS:
% These scripts implement the ensemble hydrograph separation approach as presented in J.W. Kirchner, "Quantifying new water fractions
%    and transit time distributions using ensemble hydrograph separation: theory and benchmark tests", Hydrology and Earth System Sciences, 
%    23, 303-349, 2019.  These scripts are further described in J.W. Kirchner and J.L.A. Knapp, "Matlab and R scripts for ensemble hydrograph
%    separation", Hydrology and Earth System Sciences.  
%    Users should cite both of these papers (the first for the method, and the second for the scripts)


% The equation numbers in the scripts refer to the corresponding equations in Kirchner (2019).  The equations may differ in detail.  For example,
% in these scripts, k ranges from 1 to m+1 instead of 0 to m due to the array indexing conventions of the R language.
% Also, as described below, in calculating TTD's the counts of above-threshold precipitation n.xk and n.xkxl have been
% replaced by counts of usable inputs, denoted u.xk and u.xkxl.  Usable inputs either have precipitation tracer measurements,
% or have below-threshold precipitation so that one can assume that the tracer input was negligible.  The equations are adjusted accordingly.

%% make profiles of Fnew
%(Cp, Cq, P, Q, p_threshold, crit, lwr, upr, vol_wtd, robust, ser_corr, ptfilter)
% takes as input:
% - Cp           vector of tracer concentration in precipitation (at equally spaced points in time)
% - Cq           vector of tracer concentration in streamflow (assumed to be a vector at the same time points at Cp)
% - P            vector of precipitation rate (over the same time intervals as Cp)
% - Q            optioNaNl vector of discharge (in same units as precipitation)
% - p_threshold  threshold precipitation rate below which tracer inputs are ignored
% - crit         vector containing criterion for subsetting (for example, Q or P)
% - lwr          lower bounds of subsets in percentile ranks
% - upr          upper bounds of subsets in percentile ranks
% - vol_wtd      logical flag indicating whether volume-weighted new water fractions are desired (TRUE = 1) or not (FALSE = 0)
% - robust       if TRUE (1), Fnew is estimated by iteratively reweighted least squares (a robust estimation method)
% - ser_corr     if TRUE (1), standard error estimates account for serial correlation in residuals
% - ptfilter     optional vector of flags (1 and 0, T and F, or TRUE and FALSE) indicating whether individual time steps 
%                should be included in the analysis.  Note that ptfilter is redefined internally before being passed to EHS_Fnew
%
% EHS_profile divides the input time series into length(lwr) (=length(upr)) subsets, for which the percentile 
% rank of crit is between lwr and upr.  It then runs EHS_Fnew on each of these subsets and returns vectors of the results.
%
% returns the following outputs
% - bin_avg       bin averages of criterion variable
% - QpFnew        new water fraction for time steps with precipitation
% - QpFnew_se     standard error of QpFnew
% - QFnew         new water fraction for all time steps, including periods below rainfall threshold
% - QFnew_se      standard error of QFnew
% - PFnew         forward new water fraction (fraction of precipitation that becomes streamflow in the same time step)
% - PFnew_se      standard error of forward Fnew
%
%               if vol_wtd = TRUE, QpFnew, QpFnew_se, QFnew, and QFnew_se are discharge-weighted,
%                  and PFnew and PFnew_se are precipitation-weighted

% Cp and Cq can have missing values (NaN)
% P  must not have missing values
% Q can be omitted, but in that case the forward Fnew and all weighted Fnew's will be NaN
% crit must be same length as vectors of Cp, Cq, P, and Q
%
% Be aware that in some circumstances, profiles using P as a criterion can yield strange results.  
% Specifically, filtering to include small values of P can lead to 
% small and uncertain values of QpFnew, and in some cases to
% average Q >> average P (because Q can be supplied by P in previous time steps).  
% Then the conversion to PFnew will multiply by the large ratio of
% average Q/average P, leading to large and highly uncertain values of PFnew.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin < 2       % create opts strcutrue
    opts = [];
end
if nargin < 3   % create profileopts strcutrue
    profileopts = [];
end

% extract data from structures
Cp = inputdat.Cp;
Cq = inputdat.Cq;
P = inputdat.p;
crit = profileopts.crit;


%%%%%%%%%%
% set default values
% inputdat
if isfield(inputdat,'q')   Q = inputdat.q;
else     Q = nan(size(Cp));     end

% opts
if isfield(opts,'ser_corr')     ser_corr = opts.ser_corr;
else     ser_corr = 1; end

if isfield(opts,'robust')     robust = opts.robust;
else     robust = 1;  end

if isfield(opts,'vol_wtd')     vol_wtd = opts.vol_wtd;
else     vol_wtd = 0;   end

if isfield(opts,'p_thresh')     p_threshold = opts.p_thresh;
else     p_threshold = 0;   end

% profile opts
if isfield(profileopts,'ptfilter')     ptfilter = profileopts.ptfilter;
else    ptfilter = ones(size(Cp));    end

if isfield(profileopts,'upr')     upr = profileopts.upr;
else     upr = [20,30,40,50,60,70,80,90,100,100,100];   end

if isfield(profileopts,'lwr')     lwr = profileopts.lwr;
else     lwr = [0,10,20,30,40,50,60,70,80,90,95];   end




% check input data
if length(upr) ~= length(lwr)
    error('Fatal error in EHS_profile: upper and lower threshold vectors must be same length!')
end
    
if min(upr-lwr)<=0 
	error('Fatal error in EHS_profile: upper thresholds must be above lower thresholds!')
end
    
% convert the point filter to logical
ptfilter = logical(ptfilter);

% now copy the filter criterion
crit_copy = crit;
    
% We need to filter the filter criterion itself, to exclude unusable points. These include
% points with precipitation below the precipitation threshold, or with missing Cp and Cq.
crit(P<p_threshold) = NaN;
crit(isnan(Cp)) = NaN;
crit(isnan(Cq)) = NaN;
crit(isnan(P)) = NaN;
crit (~ptfilter) = NaN;         % if the input series has been filtered (by supplying ptfilter as input), apply that filter here
    
% if we are volume-weighting, we also need to filter out criterion values 
% for which discharge is unavailable
if vol_wtd 
    crit(isnan(Q)) = NaN;
end
  
% Note that filtering the criterion variable in this way gives us (say) x%
% of the distribution of potentially usable points (since, for example,
% P<p_threshold will typically not be used).  Otherwise we risk (for
% example) a case where a range of low discharges have few (or zero!) points
% that have above-threshold P, and thus Cp, with which to calculate Fnew.
    
% Note, however, that while this approach guarantees a reasonable number of
% usable points in each range of percentages, but it also means that those
% ranges will not match the raw percentiles of (for example) P or Q.
    
% convert the percentages in lwr and upr to threshold values of the criterion variable
lwr = quantile(crit(P>=p_threshold), lwr./100);  %, NaN.rm=TRUE)
upr = quantile(crit(P>=p_threshold), upr./100);  %, NaN.rm=TRUE)

% now we need to retrieve our copy of the original criterion vector
crit = crit_copy;
    
inputd.Cp = Cp;
inputd.Cq = Cq;
inputd.p = P;
inputd.q = Q;
opt.p_thresh = p_threshold;
opt.vol_wtd = vol_wtd;
opt.robust = robust;
opt.ser_corr = ser_corr;

for i = 1:length(lwr)  % step through levels of the filters
      
      % Here we re-define the point filter 
      ptfilter = ones(size(Cp));
      ptfilter(isnan(crit)) = 0;    % this is redundant given that NaN values of crit will trigger both of the next tests anyhow
      ptfilter(crit<lwr(i)) = 0;    % exclude values below lower threshold
      ptfilter(crit>upr(i)) = 0;    % exclude values above upper threshold
      ptfilter = logical(ptfilter);
      Fnewopt.ptfilter = ptfilter;
      % calculate new water fractions using the filtered data
      Fnew = EHS_Fnew(inputd,opt,Fnewopt); 

      % put upper and lower limits into data frame
      EHSoutput.lwr_limit(i,1) = lwr(i);
      EHSoutput.upr_limit(i,1) = upr(i);
      
      % average criterion value within this bin
      EHSoutput.bin_avg(i,1) = nanmean(crit(ptfilter));
      % average criterion value within this bin for points with above-threshold precipitation
      EHSoutput.bin_avg_with_P(i,1) = nanmean(crit(ptfilter & P>=p_threshold));
      
      % make the results into a new row of a data frame
      EHSoutput.QpFnew(i,1) = Fnew.QpFnew;
      EHSoutput.QpFnew_se(i,1) = Fnew.QpFnew_se;
      EHSoutput.QFnew(i,1) = Fnew.QFnew;
      EHSoutput.QFnew_se(i,1) = Fnew.QFnew_se;
      EHSoutput.PFnew(i,1) = Fnew.PFnew;
      EHSoutput.PFnew_se(i,1) = Fnew.PFnew_se;

end
    
end
